@@ -18,12 +18,12 @@ import ast
 import builtins
 import importlib.util
 import sys
-from types import ModuleType, CodeType
+from types import CodeType, ModuleType
 
-from .dialects import DialectExpander
-from .expander import find_macros, expand_macros
-from .markers import check_no_markers_remaining
 from . import multiphase
+from .dialects import DialectExpander
+from .expander import expand_macros, find_macros
+from .markers import check_no_markers_remaining
 from .unparser import unparse
 from .utils import gensym, getdocstring
 
@@ -48,9 +48,8 @@ def expand(source, filename, optimize=-1, self_module=None):
 
     `filename`:     Full path to the `.py` file being compiled.
 
-    `optimize`:     Passed to Python's built-in `compile` function, as well as to
-                    the multi-phase compiler. The multi-phase compiler uses the
-                    `optimize` setting for the temporary higher-phase modules.
+    `optimize`:     Passed to the multi-phase compiler. It uses the `optimize`
+                    setting for the temporary higher-phase modules.
 
     `self_module`:  Absolute dotted module name of the module being compiled.
                     Needed for modules that request multi-phase compilation.
@@ -193,6 +192,8 @@ def compile(source, filename, optimize=-1, self_module=None):
     The main reason for its existence is to provide a near drop-in replacement for
     the built-in `compile` for macro-enabled input.
 
+    The `optimize` parameter is passed to Python's built-in `compile` function.
+
     Currently the API differs from the built-in `compile` in that:
 
      - `mode` is always `"exec"`,
