@@ -6,7 +6,7 @@ __all__ = ["Dialect", "DialectExpander"]
 import ast
 import functools
 import re
-from sys import stderr
+import sys
 
 from .colorizer import setcolor, colorize, ColorScheme
 from .coreutils import ismacroimport, get_macros
@@ -114,7 +114,7 @@ class Dialect:
         return NotImplemented
 
 
-_message_header = colorize("**StepExpansion: ", ColorScheme.HEADING)
+_message_header = colorize("**StepExpansion: ", ColorScheme.HEADING1)
 class StepExpansion(Dialect):  # actually part of public API of mcpyrate.debug, for discoverability
     """[dialect] Show each step of expansion while dialect-expanding the module.
 
@@ -130,17 +130,29 @@ class StepExpansion(Dialect):  # actually part of public API of mcpyrate.debug,
     This dialect has no other effects.
     """
     def transform_source(self, text):
-        self.expander.debugmode = True
-        c, CS = setcolor, ColorScheme
-        msg = f"{c(CS.SOURCEFILENAME)}{self.expander.filename} {c(CS.HEADING)}enabled {c(CS.ATTENTION)}DialectExpander debug mode {c(CS.HEADING)}while taking step {self.expander._step + 1}.{c()}"
-        print(_message_header + msg, file=stderr)
-        # Pass through the input (instead of returning `NotImplemented`) to
+        # We pass through the input (instead of returning `NotImplemented`) to
         # consider this as having taken a step, thus triggering the debug mode
         # output printer. (If this was the first dialect applied, our output is
         # actually the original input; but there's no way to know to show it
         # before this dialect has run.)
+        self._enable_debugmode()
         return text
 
+    # This exists only so that dialect-enabled AST compiles can use `StepExpansion`, too.
+    def transform_ast(self, tree):
+        # If the debug mode was already enabled, we're ok - behave as if this method didn't exist
+        # (to suppress the debug printout of a step that did nothing).
+        if self.expander.debugmode:
+            return NotImplemented
+        self._enable_debugmode()
+        return tree
+
+    def _enable_debugmode(self):
+        self.expander.debugmode = True
+        c, CS = setcolor, ColorScheme
+        msg = f"{c(CS.SOURCEFILENAME)}{self.expander.filename} {c(CS.HEADING1)}enabled {c(CS.ATTENTION)}DialectExpander debug mode {c(CS.HEADING1)}while taking step {self.expander._step + 1}.{c()}"
+        print(_message_header + msg, file=sys.stderr)
+
 # --------------------------------------------------------------------------------
 
 _dialectimport = re.compile(r"^from\s+([.0-9a-zA-z_]+)\s+import dialects,\s+([^(\\]+)\s*$",
@@ -198,9 +210,9 @@ class DialectExpander:
         c, CS = setcolor, ColorScheme
         if self.debugmode:
             plural = "s" if self._step != 1 else ""
-            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING)}before dialect {c(CS.TRANSFORMERKIND)}{kind} {c(CS.HEADING)}transformers ({self._step} step{plural} total):{c()}\n"
-            print(_message_header + msg, file=stderr)
-            print(format_for_display(content), file=stderr)
+            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING1)}before dialect {c(CS.TRANSFORMERKIND)}{kind} {c(CS.HEADING1)}transformers ({self._step} step{plural} total):{c()}\n"
+            print(_message_header + msg, file=sys.stderr)
+            print(format_for_display(content), file=sys.stderr)
 
         # We collect and return the dialect object instances so that both
         # `transform_ast` and `postprocess_ast` can use the same instances. The
@@ -250,14 +262,14 @@ class DialectExpander:
                 self._step += 1
 
                 if self.debugmode:
-                    msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING)}after {c(CS.DIALECTTRANSFORMERNAME)}{module_absname}.{dialectname}.{transform} {c(CS.HEADING)}(step {self._step}):{c()}\n"
-                    print(_message_header + msg, file=stderr)
-                    print(format_for_display(content), file=stderr)
+                    msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING1)}after {c(CS.DIALECTTRANSFORMERNAME)}{module_absname}.{dialectname}.{transform} {c(CS.HEADING1)}(step {self._step}):{c()}\n"
+                    print(_message_header + msg, file=sys.stderr)
+                    print(format_for_display(content), file=sys.stderr)
 
         if self.debugmode:
             plural = "s" if self._step != 1 else ""
-            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING)}completed all dialect {c(CS.TRANSFORMERKIND)}{kind} {c(CS.HEADING)}transforms ({self._step} step{plural} total).{c()}"
-            print(_message_header + msg, file=stderr)
+            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING1)}completed all dialect {c(CS.TRANSFORMERKIND)}{kind} {c(CS.HEADING1)}transforms ({self._step} step{plural} total).{c()}"
+            print(_message_header + msg, file=sys.stderr)
 
         return content, dialect_instances
 
@@ -271,9 +283,9 @@ class DialectExpander:
         c, CS = setcolor, ColorScheme
         if self.debugmode:
             plural = "s" if self._step != 1 else ""
-            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING)}before dialect {c(CS.TRANSFORMERKIND)}AST postprocessors {c(CS.HEADING)}({self._step} step{plural} total):{c()}\n"
-            print(_message_header + msg, file=stderr)
-            print(format_for_display(tree), file=stderr)
+            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING1)}before dialect {c(CS.TRANSFORMERKIND)}AST postprocessors {c(CS.HEADING1)}({self._step} step{plural} total):{c()}\n"
+            print(_message_header + msg, file=sys.stderr)
+            print(format_for_display(tree), file=sys.stderr)
 
         content = tree
         for dialect in dialect_instances:
@@ -296,14 +308,14 @@ class DialectExpander:
             self._step += 1
 
             if self.debugmode:
-                msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING)}after {c(CS.DIALECTTRANSFORMERNAME)}{format_macrofunction(dialect)}.postprocess_ast {c(CS.HEADING)}(step {self._step}):{c()}\n"
-                print(_message_header + msg, file=stderr)
-                print(format_for_display(content), file=stderr)
+                msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING1)}after {c(CS.DIALECTTRANSFORMERNAME)}{format_macrofunction(dialect)}.postprocess_ast {c(CS.HEADING1)}(step {self._step}):{c()}\n"
+                print(_message_header + msg, file=sys.stderr)
+                print(format_for_display(content), file=sys.stderr)
 
         if self.debugmode:
             plural = "s" if self._step != 1 else ""
-            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING)}completed all dialect {c(CS.TRANSFORMERKIND)}AST postprocessors {c(CS.HEADING)}({self._step} step{plural} total).{c()}"
-            print(_message_header + msg, file=stderr)
+            msg = f"{c(CS.SOURCEFILENAME)}{self.filename} {c(CS.HEADING1)}completed all dialect {c(CS.TRANSFORMERKIND)}AST postprocessors {c(CS.HEADING1)}({self._step} step{plural} total).{c()}"
+            print(_message_header + msg, file=sys.stderr)
 
         return content
 
