@@ -52,7 +52,7 @@ __all__ = ["namemacro", "isnamemacro",
            "expand_macros", "find_macros"]
 
 import sys
-from ast import (AST, Assign, Call, Constant, Import, Lambda, Name,
+from ast import (AST, Assign, Call, Starred, Constant, Import, Lambda, Name,
                  NodeVisitor, Store, Subscript, Tuple, alias, arguments,
                  copy_location, iter_fields)
 from copy import copy
@@ -96,7 +96,7 @@ def isparametricmacro(function):
 
 # --------------------------------------------------------------------------------
 
-def destructure_candidate(tree):
+def destructure_candidate(tree, *, _validate_call_syntax=True):
     """Destructure a macro call candidate AST, `macroname` or `macroname[arg0, ...]`."""
     if type(tree) is Name:
         return tree.id, []
@@ -111,6 +111,24 @@ def destructure_candidate(tree):
         else:  # anything that doesn't have at least one comma at the top level
             macroargs = [macroargs]
         return tree.value.id, macroargs
+    # Up to Python 3.8, decorators cannot be subscripted. This is a problem for
+    # decorator macros that would like to have macro arguments.
+    #
+    # To work around this, we allow passing macro arguments also using parentheses
+    # (like in `macropy`). Note macro arguments must still be passed positionally!
+    #
+    # For uniformity, we limit to a subset of the function call syntax that
+    # remains valid if you replace the parentheses with brackets in Python 3.9.
+    elif type(tree) is Call and type(tree.func) is Name and not tree.keywords:
+        # `MacroExpander._detect_macro_items` needs to perform a preliminary check
+        # without full validation when it is trying to detect which `with` items
+        # and decorators are in fact macro invocations.
+        if _validate_call_syntax:
+            if not tree.args:  # reject empty args
+                raise SyntaxError("must provide at least one argument when passing macro arguments")
+            if any(type(arg) is Starred for arg in tree.args):  # reject starred items
+                raise SyntaxError("unpacking (splatting) not supported in macro argument position")
+        return tree.func.id, tree.args
     return None, None  # not a macro invocation
 
 
@@ -199,6 +217,28 @@ class MacroExpander(BaseMacroExpander):
         do anything it wants to its input tree. Any remaining block macro
         invocations are attached to the `With` node, so if that is removed,
         they will be skipped.
+
+        **NOTE**: At least up to v3.3.0, if there are three or more macro
+        invocations in the same `with` statement::
+
+            with macro1, macro2, macro3:
+                ...
+
+        this is equivalent with::
+
+            with macro1:
+                with macro2, macro3:
+                    ...
+
+        and **not** equivalent with::
+
+            with macro1:
+                with macro2:
+                    with macro3:
+                        ...
+
+        which may be important if `macro1` needs to scan for invocations of
+        `macro2` and `macro3` to work together with them.
         """
         macros, others = self._detect_macro_items(withstmt.items, "block")
         if not macros:
@@ -288,7 +328,7 @@ class MacroExpander(BaseMacroExpander):
                 candidate = item.context_expr
             else:
                 candidate = item
-            macroname, macroargs = destructure_candidate(candidate)
+            macroname, macroargs = destructure_candidate(candidate, _validate_call_syntax=False)
 
             # warn about likely mistake
             if (macroname and self.isbound(macroname) and
